#include <AmbiSenseExtension.h>
#include "TSL2561.h"
#ifdef TwoWire_h
#include "Arduino.h"


TSL2561 tsl(TSL2561_ADDR_FLOAT); 

AmbiSenseExtension::AmbiSenseExtension(){


}

void AmbiSenseExtension::initialize(){
	Wire.begin();
	tsl.begin();
  // You can change the gain on the fly, to adapt to brighter/dimmer light situations
  //tsl.setGain(TSL2561_GAIN_0X);         // set no gain (for bright situtations)
  tsl.setGain(TSL2561_GAIN_16X);      // set 16x gain (for dim situations)
  
  // Changing the integration time gives you a longer time over which to sense light
  // longer timelines are slower, but are good in very low light situtations!
  tsl.setTiming(TSL2561_INTEGRATIONTIME_13MS);  // shortest integration time (bright light)
  //tsl.setTiming(TSL2561_INTEGRATIONTIME_101MS);  // medium integration time (medium light)
  //tsl.setTiming(TSL2561_INTEGRATIONTIME_402MS);  // longest integration time (dim light)
}

int AmbiSenseExtension::getVisible(void){
  int visible = tsl.getLuminosity(TSL2561_VISIBLE);   
  return visible;
}

int AmbiSenseExtension::getInfrared(void){
  int infrared = tsl.getLuminosity(TSL2561_INFRARED);   
  return infrared;
}

int AmbiSenseExtension::getFullSpectrum(void){
  int full = tsl.getLuminosity(TSL2561_FULLSPECTRUM);
  return full;
}

int AmbiSenseExtension::getVisible(int min, int max){
  int visible = tsl.getLuminosity(TSL2561_VISIBLE);   
  visible = map(visible,0,32767,min, max);
  return visible;
}

int AmbiSenseExtension::getInfrared(int min, int max){
  int infrared = tsl.getLuminosity(TSL2561_INFRARED);   
  infrared = map(infrared,0,32767,min, max);
  return infrared;
}

int AmbiSenseExtension::getFullSpectrum(int min, int max){
  int full = tsl.getLuminosity(TSL2561_FULLSPECTRUM);
  full = map(full,0,32767,min, max);
  return full;
}

float AmbiSenseExtension::getTemperature(){
	unsigned int T_dat;
	float T_C;
	T_dat = read_temperature();
	T_C = (float) (T_dat)* 0.010072 - 40.0;
	return T_C;
}

float AmbiSenseExtension::getHumidity(){
	unsigned int H_dat;
	float RH;
	H_dat = read_humidity();
	RH = (float) H_dat * 6.10e-3;
	return RH;
}

unsigned int AmbiSenseExtension::read_temperature()
{
      byte address, Hum_H, Hum_L, Temp_H, Temp_L;
      unsigned int T_dat;
      address = 0x27;;
      Wire.beginTransmission(address); 
      Wire.endTransmission();
      delay(10);
      Wire.requestFrom((int)address, (int) 4);
      Hum_H = Wire.read();
      Hum_L = Wire.read();
      Temp_H = Wire.read();
      Temp_L = Wire.read();
      Wire.endTransmission();
      T_dat = ((((unsigned int)Temp_H) << 8) | Temp_L)>>2;
			return T_dat;
}


unsigned int AmbiSenseExtension::read_humidity()
{
      byte address, Hum_H, Hum_L, Temp_H, Temp_L;
      unsigned int H_dat;
      address = 0x27;;
      Wire.beginTransmission(address); 
      Wire.endTransmission();
      delay(10);
      
      Wire.requestFrom((int)address, (int) 4);
      Hum_H = Wire.read();
      Hum_L = Wire.read();
      Temp_H = Wire.read();
      Temp_L = Wire.read();
      Wire.endTransmission();
     
      Hum_H = Hum_H & 0x3f;
      H_dat = (((unsigned int)Hum_H) << 8) | Hum_L;

      return H_dat;
      
}
#endif
