/*
 * BRIX2.cpp - Library for BRIX2 modules
 * Sebastian Zehe, 2013
 */

#include <BRIX2.h>

BRIX2::BRIX2(){

}

// System Access
void BRIX2::initialize(){
  // setup LED pins
  pinMode(LED_RED, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BLUE, OUTPUT);
  // set LED pins high, all LEDs off
  digitalWrite(LED_RED, HIGH);
  digitalWrite(LED_GREEN, HIGH);
  digitalWrite(LED_BLUE, HIGH);  
  // define color gradients

}

void BRIX2::resetSystemController(){
	pinMode(RESET_PIN, OUTPUT);
	digitalWrite(RESET_PIN,LOW);
	delay(20);
	pinMode(RESET_PIN, INPUT);
}

void BRIX2::statusLedOn(){
	Serial1.write(0x02);
	Serial1.write(LOCALCALL);
	Serial1.write(0x02);
	Serial1.write(0x01);
	Serial1.write(0x0D);
}

void BRIX2::statusLedOff(){  
	Serial1.write(0x02);
	Serial1.write(LOCALCALL);
	Serial1.write(0x03);
	Serial1.write(0x01);
	Serial1.write(0x0D);
}

void BRIX2::RXTXOff(){  
	Serial1.write(0x02);
	Serial1.write(LOCALCALL);
	Serial1.write(0x05);
	Serial1.write(0x01);
	Serial1.write(0x0D);
}

void BRIX2::setRGB(unsigned int R, unsigned int G, unsigned int B){
  analogWrite(LED_RED,255-R);
  analogWrite(LED_GREEN,255-G);
  analogWrite(LED_BLUE,255-B);
}

void BRIX2::setHSV(unsigned int hue, unsigned int sat, unsigned int val){
        unsigned char r,g,b;
        unsigned int H_accent = hue/60;
        unsigned int bottom = ((255 - sat) * val)>>8;
        unsigned int top = val;
        unsigned char rising  = ((top-bottom)  *(hue%60   )  )  /  60  +  bottom;
        unsigned char falling = ((top-bottom)  *(60-hue%60)  )  /  60  +  bottom;

        switch(H_accent) {
        case 0:
                r = top;
                g = rising;
                b = bottom;
                break;

        case 1:
                r = falling;
                g = top;
                b = bottom;
                break;

        case 2:
                r = bottom;
                g = top;
                b = rising;
                break;

        case 3:
                r = bottom;
                g = falling;
                b = top;
                break;

        case 4:
                r = rising;
                g = bottom;
                b = top;
                break;

        case 5:
                r = top;
                g = bottom;
                b = falling;
                break;
        }
        setRGB(r,g,b);
}

float BRIX2::getVoltage(){
  double actualVoltage = 0.0;
  int voltage;
  // Send Request Voltage command
  Serial1.write(0x03);
  Serial1.write(LOCALCALL);
  Serial1.write(0x01);
  Serial1.write(0x01);
  Serial1.write(0x0D);
  char inbyte;
  String incomingBuffer = "0000000";
  char bufPosition=0; 
  while(Serial1.available()>0) {
    inbyte = Serial1.read();
    if(inbyte=='\n') {
      // Newline detected
      incomingBuffer[bufPosition-1]='\0'; // NULL terminate the string
      break;
    }
    else {        
      incomingBuffer[bufPosition]=(char)inbyte;
      bufPosition++;
      if(bufPosition>4) {
        bufPosition=0;
      }
    }
  }
  voltage = incomingBuffer.toInt();
  actualVoltage = (((voltage/1024.0)*3.3)/10)*32;
  return actualVoltage;
}

// RF Stuff
int BRIX2::getAddress(){
 int address = 0x00;
  // Send Request Address command
  Serial1.write(0x03);
  Serial1.write(LOCALCALL);
  Serial1.write(0x03);
  Serial1.write(0x01);
  Serial1.write(0x0D);
	char inbyte;
  String incomingBuffer = "00000";
  char bufPosition=0; 
  delay(10);
  while(Serial1.available()>0) {
    inbyte = Serial1.read();
    delay(10);
    if(inbyte=='\n') {
      // Newline detected
      incomingBuffer[bufPosition-1]='\0'; // NULL terminate the string
      break;
    }
    else {        
      incomingBuffer[bufPosition]=(char)inbyte;
      bufPosition++;
      if(bufPosition>4) {
        bufPosition=0;
      }
    }
  }
  address = incomingBuffer.toInt();
  return address;
}

boolean BRIX2::checkNode(byte address, int timeOut){
  boolean isPresent = false;
  return isPresent;
}

void BRIX2::sendAscii(byte address, char* text, byte length){
  if (length<58){
    // Write header 
    Serial1.write(0x01); // RF packet
    Serial1.write(address);
    Serial1.write(0x01); // ASCII packet type
    Serial1.write(length-1);
    for (int i = 0;i<length-1;i++){
      Serial1.write(text[i]);
    }
    Serial1.write('\r');

  }
}

void BRIX2::sendInt(byte address, int * numbers, byte length){
  if (length<58){
    // Write header 
    Serial1.write(0x01); // RF packet
    Serial1.write(address);
    Serial1.write(0x02); // Int packet type
    Serial1.write(length);
    for (uint8_t i = 0;i<length-1;i++){
      String number = String(numbers[i]);
      for (uint8_t j = 0;j<number.length();j++){
        Serial1.write(number.charAt(j));
      }
      Serial1.write(',');
    }
    String number = String(numbers[length-1]);
    for (uint8_t j = 0;j<number.length();j++){
      Serial1.write(number.charAt(j));
    }
    Serial1.write('\r');
  }
}

void BRIX2::sendFloat(byte address, int * numbers, byte length){
}



