////////////////////////////////////////////////////////
// AccelerometerVolumeControl,                        //
// User Application,                                  //
// This example demonstrates how to control a Samsung //
// TV volume by tilting the BRIX₂ Module.             //
// The top side of the module faces the TV, the USB   //
// port points down. A Proximity Extension is used as // 
// a button and needs to be covered with a finger to  //
// activate the signal.                               //
////////////////////////////////////////////////////////

#include <Wire.h> // If you need Wire.h ALWAYS include it first!
#include <BRIX2.h>
#include <InertialSensor.h>
#include <IRremote.h>
#include <ProximityExtension.h>

// Generate Sensor Objects and BRIX₂ Object
InertialSensor mySensor;
ProximityExtension myProx;
BRIX2 myBrick;

IRsend irsend;

// code for volume up
unsigned int S_vup[68]={
  4600,4350,650,1550,700,1500,700,1550,700,400,700,400,700,450,650,450,700,400,700,1500,700,1550,650,1550,700,400,700,400,700,450,650,450,700,400,700,1500,700,1550,650,1550,700,400,700,450,700,400,700,400,700,400,700,450,650,450,650,450,650,1550,700,1500,700,1550,700,1500,700,1550,650};
// code for volume down
unsigned int S_vdown[68]={
  4600,4350,700,1550,650,1550,700,1500,700,450,650,450,700,400,700,400,700,400,700,1550,700,1500,700,1550,700,400,700,400,700,400,700,450,650,450,650,1550,700,1500,700,450,650,1550,700,400,700,400,700,450,700,400,700,400,700,400,700,1550,700,400,700,1500,700,1500,700,1550,700,1500,700};

void setup()
{
  myBrick.initialize();
  mySensor.initialize();
  myProx.initialize();
}

void loop() {

  // get sensor data
  int ay = mySensor.getAccelerationY(-200,200);
  int proximity = myProx.getProximity(0,360);

  if(proximity>359){
    // determine if module is tilted right
    if (ay > 10){
      if (ay > 100) ay = 100;
      //volume up. the more you turn, the shorter the delay between sends
      irsend.sendRaw(S_vdown,68,38);
      delay(120-abs(ay));
    }
    // determine if module is tilted left
    if (ay < -10){
      //volume down
      if (ay < -100) ay = -100;
      irsend.sendRaw(S_vup,68,38);
      delay(120-abs(ay));
    }
  }
}

