///////////////////////////////////////////////////////
// LightShow,                                        //
// User Application,                                 //
// This example demonstrates how read data from an   //
// SD Card using the SD Extension. The data is used  //
// to control the RGB LED                            //
///////////////////////////////////////////////////////


/*
Example Color File:
====================
7,249,0
252,14,0
2,0,251
248,1,255
2,250,255
252,255,3
148,249,255
120,123,0
130,0,128
122,0,5
255,126,122
133,128,255
122,255,120
103,103,0

====================
(without the ===, and YES, copy the empty line too!)
*/

#include <BRIX2.h>
// Below are the includes required for the SD Card
#include <SD.h>
#include <SPI.h>

// Generate BRIX₂ Object and File Object
BRIX2 myBrick;
File dataFile;

int red,green,blue;

void setup() {
  // Initialize ALL the things!
  myBrick.initialize();
  // Begin SD, Chip Select on Pin 4 (default on the SD Extension)
  SD.begin(4);
  Serial.begin(9600);
}

void loop() {
  // The file name you use must not exceed 8 characters plus file extension!
  dataFile = SD.open("LITESHOW.TXT");
  char currentByte = '0';
  String collector = "";
  while (dataFile.available()) {
    // Collect bytes until not a number (for example a ",") is printed
    while (currentByte > 47){
      collector += currentByte;
      currentByte = dataFile.read();
    }
    // Convert the collected characters to a number that represents red
    red = collector.toInt();
    currentByte = '0';
    collector = "";
    // Collect bytes until not a number (for example a ",") is printed
    while (currentByte > 47){
      collector += currentByte;
      currentByte = dataFile.read();
    }
    // Convert the collected characters to a number that represents green
    green = collector.toInt();
    currentByte = '0';
    collector = "";
    // Collect bytes until not a number (for example a ",") is printed
    while (currentByte > 47){
      collector += currentByte;
      currentByte = dataFile.read();
    }
    // Convert the collected characters to a number that represents blue
    blue = collector.toInt();
    currentByte = '0';
    collector = "";
    // Read the extra character at the end of the line
    dataFile.read();
    // Set the LED color
    myBrick.setRGB(red,green,blue);
      delay(500);
  }
  // Close the file:
  dataFile.close();
}



// Processing (https://www.processing.org/download/) sketch 
// to generate color files for your SD Card.
//
// Move the cursor in the window and click at the color you like.
// The program will print the color values in the processing main window.
// When you are finished recording your sequence, copy those color values
// into your LITESHOW.TXT on the SD card. 
// IMPORTANT: hit "Enter" once at the end of the file!

/*


void setup() 
{
  size(800, 800);
  colorMode(HSB, 800, 400, 400);  
  noStroke();
}

void draw() 
{
  background(0);
  for (int i = 0;i<800;i++) {
    for (int j = 0;j<400;j++) {
      set(i, j, color(i, j, 400));
    }
  }
  for (int i = 0;i<800;i++) {
    for (int j = 400;j<800;j++) {
      set(i, j, color(i, 400, 800-j));
    }
  }
  color c = get(mouseX, mouseY);
  fill(c);
  rect(mouseX-20, mouseY-20, 40, 40);
}

void mouseClicked() {
  color c = get(mouseX, mouseY);
  int red = int(map(int(red(c)), 0, 800, 0, 255));
  int green = int(map(int(green(c)), 0, 400, 0, 255));
  int blue = int(map(int(blue(c)), 0, 400, 0, 255));
  println(red+","+green+","+blue);
}

*/


