///////////////////////////////////////////////////////
// SensorLogger,                                     //
// User Application,                                 //
// This example demonstrates how write data from the //
// BRIX_2 motion sensor to an SD Card using the      //
// SD Extension                                      //
///////////////////////////////////////////////////////

#include <Wire.h> // If you need Wire.h ALWAYS include it first!
#include <InertialSensor.h>
#include <BRIX2.h>
// below are the includes required for the SD Card
#include <SD.h>
#include <SPI.h>

// Generate Sensor Object, BRIX₂ Object and File Object
InertialSensor mySensor;
BRIX2 myBrick;
File dataFile;

void setup() {
  // initialize ALL the things!
  myBrick.initialize();
  mySensor.initialize();
  // begin SD, Chip Select on Pin 4 (default on the SD Extension)
  SD.begin(4);
}

void loop() {
  // Read accelerometer/gyroscope measurements
  int ax = mySensor.getAccelerationX();
  int ay = mySensor.getAccelerationY();
  int az = mySensor.getAccelerationZ();
  int gx = mySensor.getRotationX();
  int gy = mySensor.getRotationY();
  int gz = mySensor.getRotationZ();

  // Red LED indicates writing process. Let's turn it on before we write
  myBrick.setRGB(255,0,0);

  // Open the file "datalog.txt" on the card and write into it
  // The file name you use must not exceed 8 characters plus file extension!
  dataFile = SD.open("datalog.txt", FILE_WRITE);

  dataFile.print("Acc:");
  dataFile.print(ax);
  dataFile.print(",");
  dataFile.print(ay);
  dataFile.print(",");
  dataFile.print(az);
  dataFile.print("  Gyr:");
  dataFile.print(gx);
  dataFile.print(",");
  dataFile.print(gy);
  dataFile.print(",");
  dataFile.println(gz);

  dataFile.close();

  // Turn LED off again
  myBrick.setRGB(0,0,0);

  delay(100);
}


