///////////////////////////////////////////////
// Tetris,                                   //
// User Application,                         //
// Shows how to use the VibroSound Extension //
// to create simple melodies.                //
//                                           //
// Based on the note-Example,                //
// created 21 Jan 2010                       //
// modified 30 Aug 2011                      //
// by Tom Igoe                               //
// http://arduino.cc/en/Tutorial/Tone        //
///////////////////////////////////////////////

#include <BRIX2.h>
#include <VibroSoundExtension.h>
#include "pitches.h"

BRIX2 myBrick;
VibroSoundExtension myBuzzer;
// Notes taken from:
// https://www.chaos-inkl.de/wiki/project:arduino-kickstarter-tetris

int melody[] = {
  NOTE_E3, NOTE_H2, NOTE_C3, NOTE_D3, NOTE_E3, NOTE_D3, NOTE_C3, NOTE_H2,
  NOTE_A2, NOTE_A2, NOTE_C3, NOTE_E3, NOTE_D3, NOTE_C3,
  NOTE_H2, NOTE_H2, NOTE_C3, NOTE_D3, NOTE_E3,
  NOTE_C3, NOTE_A2, NOTE_A2,
  0, NOTE_D3, NOTE_F3, NOTE_A3, NOTE_G3, NOTE_F3,
  NOTE_E3, NOTE_E3, NOTE_C3, NOTE_E3, NOTE_D3, NOTE_C3,
  NOTE_H2, NOTE_H2, NOTE_C3, NOTE_D3, NOTE_E3,
  NOTE_C3, NOTE_A2, NOTE_A2, 0,
  NOTE_E2, NOTE_C2, NOTE_D2, NOTE_H1, NOTE_C2, NOTE_A1, NOTE_GS1, NOTE_H1,
  NOTE_E2, NOTE_C2, NOTE_D2, NOTE_H1, NOTE_C2, NOTE_E2, NOTE_A2, NOTE_GS2
};
  
// note durations: 4 = quarter note, 8 = eighth note, etc.:
int noteDurations[] = {
  4,8,8,8,16,16,8,8,
  4,8,8,4,8,8,
  4,8,8,4,4,
  4,4,2,
  8,4,8,4,8,8,
  4,8,8,4,8,8,
  4,8,8,4,4,
  4,4,4,4,
  2,2,2,2,2,2,2,2,
  2,2,2,2,4,4,2,1
};//59

void setup() {
  myBrick.initialize();
  myBuzzer.initialize();
}

void loop() {
  // iterate over the notes of the melody:
  for (int thisNote = 0; thisNote < 59; thisNote++) {
    // to calculate the note duration, take one second 
    // divided by the note type.
    //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
    int noteDuration = 1000/noteDurations[thisNote];
    myBuzzer.Beep(melody[thisNote],noteDuration);
    // to distinguish the notes, set a minimum time between them.
    // the note's duration + 30% seems to work well:
    int pauseBetweenNotes = noteDuration * 1.30;
    delay(pauseBetweenNotes);
    // stop the tone playing:
    myBuzzer.noBeep();
  }
}

