//////////////////////////////////////////////////////
// RF Clone                                         //           
// User Application                                 //
// This example demonstrates how to communicate     //
// between two BRIX via RF.                         //
// The sender changes the LED color if you put it   // 
// flat on one of its sides. That LED state is      //
// sent to the receiver which clones it.            //
//                                                  //
// This sketch can be used for receiver AND sender, //
// depending on #define ROLE.                       //
//////////////////////////////////////////////////////

#include <Wire.h> // If you need Wire.h ALWAYS include it first!
#include <InertialSensor.h>
#include <BRIX2.h>

//#define ROLE RECEIVER
#define ROLE SENDER

// Generate Sensor Object and BRIX₂ Object
InertialSensor mySensor;
BRIX2 myBrick;

// string receive variables
char inbyte;
String incomingBuffer = "00000000000000000000";
static char bufPosition=0;
boolean finished = false;
int value[] = {
  0,0,0};
int pos = 0;
// role definitions. don't touch!
#define SENDER 1
#define RECEIVER 0


void setup(){
  // it is important to start Serial1 in order
  // to use the RF capabilities of the BRIX2 library
  Serial.begin(38400);
  Serial1.begin(38400);
  myBrick.initialize();
  mySensor.initialize();
}
void loop(){

  if(ROLE==SENDER){
    // get motion data
    value[0] =mySensor.getAccelerationX();
    value[1] = mySensor.getAccelerationY();
    value[2] = mySensor.getAccelerationZ();
    // debugging output
    Serial.print(value[0]);
    Serial.print(",");
    Serial.print(value[1]);
    Serial.print(",");
    Serial.println(value[2]);
    // determine on which side the modul lies
    determinePos(value);
    // reuse the value array and put the position into it
    value[0] = pos;
    // match the color to the orientation
    matchcolor(pos);  
    // send the position data array (only the first value)
    myBrick.sendInt(0x0A,value,1);
    delay(100);
  }

  if(ROLE==RECEIVER){
    // get data via RF
    if(Serial1.available()>0){
      inbyte = Serial1.read();
      if(inbyte=='\r') {
        // Newline detected
        incomingBuffer[bufPosition]='\0'; // NULL terminate the string
        finished = true;
      }
      else {
        incomingBuffer[bufPosition]=(char)inbyte;
        bufPosition++;
        if(bufPosition>30) {
          bufPosition=0;
        }
      }
    }
    // if value is received, match the color to the position
    if (finished){
      bufPosition = 0;
      finished= false;
      matchcolor(incomingBuffer.toInt());
    }
  }  
}


// determine on which side the module lies
// by 3 acceleration axes. put that side into
// the pos variable.
void determinePos(int * values){
  pos = 0;
  if((values[0]>15000)){
    pos = 1;
  }
  else if((values[1]>15000)){
    pos = 2;
  }
  else if((values[2]>15000)){
    pos = 3;
  }
  else if((values[0]< -15000)){
    pos = 4;
  }
  else if((values[1]< -15000)){
    pos = 5;
  }
  else if((values[2]< -13000)){
    pos = 6;
  }
}

// set led color according to 
// side the module lies on.
void matchcolor(int pos){
  int red = 0;
  int green = 0;
  int blue = 0;
  switch (pos) {
  case 1:
    red = 255;
    break;
  case 2:
    green= 255;
    break;
  case 3:
    blue= 255;
    break;
  case 4:
    red=255;
    green= 255;
    break;
  case 5:
    green= 255;
    blue = 255;
    break;
  case 6:
    red=255;
    blue= 255;
    break;
  }
  myBrick.setRGB(red,green,blue);
}















