#include <ProximityExtension.h>

#ifdef TwoWire_h
//#
#include "Arduino.h"


ProximityExtension::ProximityExtension(){

}

void ProximityExtension::initialize(){
	Wire.begin();  
  // configure the sensor
  writeByte(AMBIENT_PARAMETER, 0x0F);  // Single conversion mode, 128 averages
  writeByte(IR_CURRENT, 20);  // Set IR current to 200mA
  writeByte(PROXIMITY_FREQ, 2);  // 781.25 kHz
  writeByte(PROXIMITY_MOD, 0x81);  // 129, recommended by Vishay
  }

unsigned int ProximityExtension::getProximity (){
	return readProximity();
}

int ProximityExtension::getProximity(int min, int max){
  unsigned int proximityValue = readProximity();
  // check if value exceeds the limits of our mapping
  if (proximityValue < 0) proximityValue = 25000;
  if (proximityValue > 25000) proximityValue = 25000;
  if (proximityValue < 2500)  proximityValue = 2500;
  // map the values
  proximityValue = map(proximityValue, 2500,25000,min, max);
  return proximityValue;
}

// readProximity() returns a 16-bit value from the VCNL4000's proximity data registers
unsigned int ProximityExtension::readProximity()
{
  unsigned int data;
  unsigned char temp;
  temp = readByte(COMMAND_0);
  writeByte(COMMAND_0, temp | 0x08);  // command the sensor to perform a proximity measure
  while(!(readByte(COMMAND_0)&0x20)) 
    ;  // Wait for the proximity data ready bit to be set
  data = readByte(PROXIMITY_RESULT_MSB) << 8;
  data |= readByte(PROXIMITY_RESULT_LSB);
  return data;
}

// writeByte(address, data) writes a single byte of data to address
void ProximityExtension::writeByte(unsigned char address, unsigned char data)
{
  Wire.beginTransmission(VCNL4000_ADDRESS);
  Wire.write(address);
  Wire.write(data);
  Wire.endTransmission();
}

// readByte(address) reads a single byte of data from address
unsigned char ProximityExtension::readByte(unsigned char address)
{
  unsigned char data;
  Wire.beginTransmission(VCNL4000_ADDRESS);
  Wire.write(address);
  Wire.endTransmission();
  Wire.requestFrom(VCNL4000_ADDRESS, 1);
  while(!Wire.available());
  data = Wire.read();
  return data;
}

#endif
