/*
  Serial Traverse
 Sets the User Controller to a passthrough mode which routes
 the USB serial directly to the system controller. 
 This allows to use the RF module directly on a PC or to program
 the System Controller via bootloader. (See Wiki)
 */

static long baud = 57600;
static long newBaud = baud;
// LEDs
int R = 10;
int G = 11;
int B = 13;
// Responsible for turning the LEDs off again
int ledCounter = 0;
int ledPeriod = 100;

#define LINESTATE_DTR  1
#define DTR_PIN 12

void lineCodingEvent(long baud, byte databits, byte parity, byte charFormat)
{
  newBaud = baud;
}

void lineStateEvent(unsigned char linestate)
{
  if(linestate & LINESTATE_DTR)
    digitalWrite(DTR_PIN, HIGH);
  else
    digitalWrite(DTR_PIN, LOW);
}

void setup() {
  // Setup LEDs
  pinMode(R, OUTPUT);     
  pinMode(G, OUTPUT);   
  pinMode(B, OUTPUT); 
  // All OFF
  digitalWrite(R, HIGH);
  digitalWrite(G, HIGH);
  digitalWrite(B, HIGH);
  // Setup Serial ports
  Serial.begin(baud);
  Serial1.begin(baud);
  // Set DTR Pin
  pinMode(DTR_PIN, OUTPUT);
  digitalWrite(DTR_PIN, LOW);
}

void loop() {
  ledCounter ++;
  if (ledCounter > ledPeriod){
    ledCounter = 0;
    digitalWrite(R, HIGH);
    digitalWrite(B, HIGH);
  }
  // Set the new baud rate
  if(newBaud != baud) {
    baud = newBaud;
    Serial1.end();
    Serial1.begin(baud);
  }
  // Copy from virtual serial line to uart and vice versa
  if (Serial.available()) {
    // Receive LED on
    digitalWrite(B, LOW);
    ledCounter = 0;
    char c = (char)Serial.read();
    Serial1.write(c);
  }
  if (Serial1.available()) {
    // Transmit LED on
    digitalWrite(R, LOW);
    ledCounter = 0;
    char c = (char)Serial1.read();
    Serial.write(c);
  }
}

