/*
 Soft Serial Traverse
 Sets the User Controller to a passthrough mode which routes
 the USB serial directly to the AudioAmp controller. 
 This allows the user to talk to the AudioAmp controller via USB
 or to program the AudioAmp controller via USB and bootloader.
 */

#include <SoftwareSerial.h>
#include <BRIX2.h>

SoftwareSerial SoftSerialPort(SCK, 11); // RX, TX
BRIX2 myBrick;
static long baud = 4800;
static long newBaud = baud;
// Responsible for turning the LEDs off again
int ledCounter = 0;
int ledPeriod = 100;

#define LINESTATE_DTR  1
#define DTR_PIN 8

void lineCodingEvent(long baud, byte databits, byte parity, byte charFormat)
{
  newBaud = baud;
}

void lineStateEvent(unsigned char linestate)
{
  if(linestate & LINESTATE_DTR)
    digitalWrite(DTR_PIN, HIGH);
  else
    digitalWrite(DTR_PIN, LOW);
}

void setup() {
  myBrick.initialize();
  // Setup Serial ports
  Serial.begin(baud);
  SoftSerialPort.begin(baud);
  //Serial1.begin(baud);
  // Set DTR Pin
  pinMode(DTR_PIN, OUTPUT);
  digitalWrite(DTR_PIN, LOW);
}

void loop() {
  ledCounter ++;
  if (ledCounter > ledPeriod){
    ledCounter = 0;
    myBrick.setRGB(0,0,0);
  }
  // Set the new baud rate
  if(newBaud != baud) {
    baud = newBaud;
    SoftSerialPort.end();
    SoftSerialPort.begin(baud);
  }
  // Copy from virtual serial line to uart and vice versa
  if (Serial.available()) {
    // Receive LED on
    myBrick.setRGB(0,0,255);
    ledCounter = 0;
    char c = (char)Serial.read();
    SoftSerialPort.write(c);
  }
  if (SoftSerialPort.available()) {
    // Transmit LED on
    myBrick.setRGB(255,0,0);
    ledCounter = 0;
    char c = (char)SoftSerialPort.read();
    Serial.write(c);
  }
}


