/**
 * Filter serial packets by packet types and choose
 * individual actions for each serial packet.
 *
 * \todo Get rid of command mode and do the job with selfdefined serial commands
 */
void handleSerialCmd(char* command)
{
  byte i, len;
  byte arrV[2];
  //CCPACKET packet;
  ATQUERY atQuery = ATQUERY_REQUEST;  
  // Data mode?
  if (serMode == SERMODE_DATA)
  {
    // The minimum command can be 2 characters
    if (strlen(command)>1){
      dMsg("Received Serial Package.");
      // Check serial packet type
      if (command[0]==RF_PACKET){ // RF Packet
        dMsg("Serial Package is RF Packet.");
        int destinationAddress = command[1];
        if (command[2]== PACKET_DATA_ASCII){
          cc1101.sendData(generateASCIIPacket(destinationAddress, command));
          dMsg("ASCII Packet sent.");
        }
        else if (command[2]== PACKET_DATA_INT){
          cc1101.sendData(generateIntPacket(destinationAddress, command));
          dMsg("Int Packet sent.");
        }
        else if (command[2]== PACKET_DATA_FLOAT){
          cc1101.sendData(generateFloatPacket(destinationAddress, command));
          dMsg("Float Packet sent.");
        }
      }
      else if(command[0]==REQUEST_PACKET){ // Request Packet
        dMsg("Serial Package is Request Packet.");
        if(command[2] == REQ_VOLTAGE){
          handleVoltageRequest(command[1]);
        }
        if(command[2] == REQ_PING){
          handlePingRequest(LOCALCALL,command[3],0x00);
        }
        if(command[2] == REQ_ADDRESS){
          handleAddressRequest();
        }
      }
      else if(command[0]==COMMAND_PACKET){ // Command Packet
        dMsg("Serial Package is Command Packet.");
        if(command[2] == CMD_REQUEST){
          cc1101.sendData(generateRequestPacket(command[1], command));
        }
        if(command[2] == CMD_LED_ON){
          handleLedOnCommand();
        }
        if(command[2] == CMD_LED_OFF){
          handleLedOffCommand();
        }
        if (command[2] == CMD_RESET){
          swReset();
        }
        if (command[2] == CMD_RXTXOFF){
          handleRXTXOffCommand();
        }
      }
    }
  }
  // Command mode?
  else  // serMode = SERMODE_COMMAND
  {
    len = strlen(command);
    if (len < 5)
    {
      // Basic attention command
      if (!strcmp(strSerial, AT_ATTENTION))
        printOK();
      // Reset modem
      else if (!strcmp(strSerial, AT_RESET))
      {
        printOK();
        swReset();
      }
      // Go to serial data mode
      else if (!strcmp(strSerial, AT_GOTO_DATAMODE))
      {
        serMode = SERMODE_DATA;
        Serial.println("OK-Data mode");
        enableINT0irq();  // Enable wireless reception interrupt
      }
    }
    // Set new value
    else 
    {
      if ((strSerial[4] == '=') && (len >= 6))
      {
        // Get new value
        i = (charToHex(strSerial[5]) << 4) & 0xF0;
        i |= charToHex(strSerial[6]) & 0x0F;
        atQuery = ATQUERY_COMMAND;
      }
      // Frequency channel
      if (!strncmp(strSerial, AT_FREQCHANNEL, 4))
      {
        if (atQuery == ATQUERY_COMMAND)
        {
          cc1101.setChannel(i, true);
          printOK();
        }
        else
          Serial.println(cc1101.channel, HEX);
      }
      // Synchronization word
      else if (!strncmp(strSerial, AT_SYNCWORD, 4))
      {
        if (atQuery == ATQUERY_COMMAND)
        {
          if ((len-5) == 4)
          {
            arrV[0] = charToHex(strSerial[5]) << 4;
            arrV[0] |= charToHex(strSerial[6]);
            arrV[1] = charToHex(strSerial[7]) << 4;
            arrV[1] |= charToHex(strSerial[8]);
            cc1101.setSyncWord(arrV, true);
            printOK();
          }
          else
            printError();
        }
        else
        {
          Serial.print(cc1101.syncWord[0], HEX);
          Serial.println(cc1101.syncWord[1], HEX);
        }
      }
      // Device address
      else if (!strncmp(strSerial, AT_DEVADDRESS, 4))
      {
        if (atQuery == ATQUERY_COMMAND)
        {
          cc1101.setDevAddress(i, true);
          printOK();
        }
        else
          Serial.println(cc1101.devAddress, HEX);
      }
      // RF Statistics
      else if (!strncmp(strSerial, AT_RFSTATISTICS, 4))
      {
        if (atQuery == ATQUERY_COMMAND)
        {
          if (i == 0)
          {
            sendRFStatistics = false;
            printOK();
          }
          else if (i == 1)
          {
            sendRFStatistics = true;
            printOK();
          }
        }
        else
          Serial.println(sendRFStatistics);
      }
      // Address check
      else if (!strncmp(strSerial, AT_ADDRCHECK, 4))

      {
        if (atQuery == ATQUERY_COMMAND)
        {
          if (i == 0)
          {
            cc1101.disableAddressCheck();
            printOK();
          }
          else if (i == 1)
          {
            cc1101.enableAddressCheck();
            printOK();
          }
        }
        else{
          Serial.println("N/A");
        }
      }
      else{
        Serial.println("Invalid Command/Parameter");
      }
    }
  }
}

/**
 * Simply print out "ERROR"
 */
void printError(){
  Serial.println("ERROR");
}

/**
 * Simply print out "OK"
 */
void printOK(){
  Serial.println("OK");
}








