CCPACKET generateASCIIPacket(byte destinationAddress, char* serialData){
  /**
   * Generates an ASCII packet from serial data.
   *
   *  The function requires a well-formed serial packet 
   *  according to the standards. The third character needs
   *  to be the number of chars in the payload string.
   */
  dMsg("Building ASCII Packet...");
  CCPACKET packet;
  int numberOfChars = serialData[3];
  if (numberOfChars < 58){
    // generate packet header
    
    if (destinationAddress == BROADCAST) destinationAddress = 0;
    packet.data[0] = destinationAddress;
    packet.data[1] = myAddress; // Sender Address
    packet.data[2] = 0x01; // ASCII Packet
    packet.data[3] = numberOfChars;// Payload size
    packet.length = strlen(serialData); //minus the serial packet type plus the sender address
    //fill data into packet
    for (int i = 4;i<strlen(serialData);i++){
      packet.data[i] = serialData[i];
    }
    return packet;
  }
  else dMsg("Error: Packet too long!");
}

CCPACKET generateIntPacket(byte destinationAddress, char* serialData){
  /**
   * Generates an INT packet from serial data.
   *
   * The function requires a well-formed serial packet 
   * according to the standards. The third character needs
   * to be the number of ints in the payload string.
   * The numbers need to be comma-separated and ASCII coded.
   */
  dMsg("Building Int Packet...");
  int numberOfInts = serialData[3];
  if(numberOfInts < 11){
    // Arrays to hold numbers
    int numbers[numberOfInts];
    byte bytes[numberOfInts*2];
    CCPACKET packet;
    // generate packet header
    if (destinationAddress == BROADCAST) destinationAddress = 0; 
    packet.data[0] = destinationAddress;
    packet.data[1] = myAddress; // Sender Address
    packet.data[2] = 0x02; // Int Packet
    packet.data[3] = numberOfInts;// Payload size
    packet.length = numberOfInts*2+4; //two bytes for each number, plus header
    // Extract ALL the ints
    stringToInts(serialData,4, numbers);
    // Turn Ints to  bytes
    for (int i = 0;i<numberOfInts;i++){
      bytes[i*2]=(numbers[i]&0xFF00)>>8;
      bytes[i*2+1]=numbers[i]&0x00FF;
    }
    for (int i = 0;i<numberOfInts*2;i++){
      packet.data[i+4] = bytes[i];
    }
    return packet;
  }
  else dMsg("Error: Packet too long!");
}

CCPACKET simpleIntPacket(byte destinationAddress, byte numberOfInts, int*numbers){
  /**
   * Generates an int package from an int array
   *
   * This function is designed to be used localy.
   * That means sending packets directly from the
   * System Controler. This does not require a serial
   * data packet, so the numbers do not need to be 
   * encoded as ASCII. Such packets can be either sent
   * via serial to the User Controller (Address 0xFF)
   * or via RF to a remote BRIX2 module (Address 0xn).
   */
  dMsg("Building Simple Int Packet...");
  if(numberOfInts < 11){
    byte bytes[numberOfInts*2];
    CCPACKET packet;
    if (destinationAddress == BROADCAST) destinationAddress = 0;
    packet.data[0] = destinationAddress;
    packet.data[1] = myAddress; // Sender Address
    packet.data[2] = 0x02; // Int Packet
    packet.data[3] = numberOfInts;// Payload size
    packet.length = numberOfInts*2+4; //two bytes for each number, plus header
    // Turn Ints to bytes
    for (int i = 0;i<numberOfInts;i++){
      bytes[i*2]=(numbers[i]&0xFF00)>>8;
      bytes[i*2+1]=numbers[i]&0x00FF;
    }
    // Put bytes into the packets
    for (int i = 0;i<numberOfInts*2;i++){
      packet.data[i+4] = bytes[i];
    }
    return packet;
  }
  else dMsg("Error: Packet too long!");
}

/**
 * Generates a float packet from serial Data
 *
 * \todo This function needs to be implemented.
 */
CCPACKET generateFloatPacket(byte destinationAddress, char* serialData){
  CCPACKET packet;
  if (destinationAddress == BROADCAST) destinationAddress = 0;
  packet.data[0] = destinationAddress;
  packet.data[1] = myAddress; // Sender Address
  packet.data[2] = 0x03; // float Packet
  packet.data[3] = serialData[3];// Payload size
  // packet.length = serialData[3]*2+4; //two bytes for each number, plus header
  return packet;
}

/**
 * Generates a request packet from serial data
 *
 * Request packages can be used to poll data from remote 
 * BRIX2 modules, for example ping requests or data requests.
 *
 * \todo Currently, only requests without arguments can be handled.
 */
CCPACKET generateRequestPacket(byte destinationAddress, char * serialData){
  dMsg("Building Simple Int Packet...");
  CCPACKET packet;
  if (destinationAddress == BROADCAST) destinationAddress = 0;
  packet.data[0] = destinationAddress;
  packet.data[1] = myAddress; // Sender Address
  packet.data[2] = 0x05; // Request Packet
  packet.data[3] = serialData[3]; //Request number
  packet.length = 4;
  return packet;
}

/**
 * Convert a string to an array of ints.
 */
void stringToInts(char * inString, int startAt, int * numbers) {

  dMsg("Converting from String to Int...");
  int numbersIndex = 0;
  String tmp;
  int length = strlen(inString);
  // Loop over whole string, ignoring the header
  for (int index = startAt;index < length;index++){
    if (inString[index] == ','){
      // Comma found
      numbers[numbersIndex]=tmp.toInt();
      numbersIndex ++;
      tmp = "";
    }
    else if ((inString[index]>0x2C)&&(inString[index]<0x3A)){
      tmp += inString[index];
      // number or '-'
    }
    else {
      break; //Error here: No number or comma or minus.
    }
  }
  // Add the last number after the comma
  numbers[numbersIndex]=tmp.toInt();
}


