/**
 * Directly answers a ping request
 * 
 * Replies with Address as well as  RSSI and LQI of the incoming request
 */

void handlePingRequest(byte destinationAddress, byte rssiTargetAddress, byte lqi){
  // local request from serial port
  // second argument is Target Address
  if (destinationAddress == LOCALCALL){
    // build request packet to other node
    Serial.println("Building ping packet...");
    CCPACKET packet;
    packet.data[0] = rssiTargetAddress;
    packet.data[1] = myAddress; // Sender Address
    packet.data[2] = 0x05; // Request Packet
    packet.data[3] = REQ_PING;// Payload size
    packet.length = 4; //minus the serial packet type plus the sender address
    cc1101.sendData(packet);
  }
  // remote request via RF
  // second Argument is rssi
  else{
    int data []={
      myAddress, rssiTargetAddress, lqi          };
    delay(10);
    cc1101.sendData(simpleIntPacket(destinationAddress, 3,data));
  }
}

/**
 * Directly answers with an int corresponding to the system voltage
 *
 * For calculating the actual voltage from the transmitted integer value,
 * look at the voltageReadout example.
 */
void handleVoltageRequest(byte sourceAddress){
  int sensorValue = 0;
  // perform 10 mesurements, calculate the average
  for (int i = 0;i<10;i++){
    sensorValue += analogRead(A0);
    delay(10);
  }
  sensorValue /=10;
  // check if request is comming from serial (LOCALCALL) or RF
  if (sourceAddress==LOCALCALL){
    Serial.println(sensorValue);
  }
  else{
    int data []={
      sensorValue            };
    cc1101.sendData(simpleIntPacket(sourceAddress, 1,data));
  } 
}

/**
 * Directly answers with an int corresponding to the module address
 *
 * Works only locally, of course.
 */

void handleAddressRequest(){
  Serial.println(myAddress);
}


/**
 * Turn the System Status LED on
 */
void handleLedOnCommand(){
  digitalWrite(A2,HIGH);
}

/**
 * Turn the System Status LED off
 */
void handleLedOffCommand(){
  digitalWrite(A2,LOW);
}

/**
 * Perform a software reset of the System Controller
 */
void swReset(void) 
{
  wdt_disable();  
  wdt_enable(WDTO_15MS);
  while (1) {
  }
}


/**
 * Switch the RX/TX pins to input
 */
void handleRXTXOffCommand(){
  Serial.end();
  pinMode(0, INPUT);
  pinMode(1, INPUT);
}


/**
 * Convert a single ASCII char to a hex number
 */
byte charToHex(byte ch)
{
  byte val;
  if (ch >= 'A' && ch <= 'F')
    val = ch - 55;
  else if (ch >= 'a' && ch <= 'f')
    val = ch - 87;
  else if (ch >= '0' && ch <= '9')
    val = ch - 48;
  else
    val = 0x00;
  return val;  
}

/**
 * Print the input string if in debugMode
 */
void dMsg(char *message){
  if (debugMode){
    Serial.println(message);
  }

}





