#ifndef _SYSTEM_SERVICES_H
#define _SYSTEM_SERVICES_H

#include "Arduino.h"
#include <avr/wdt.h>

#define SERIAL_BUF_LEN           128     // Maximum length for any serial string
#define TIMER1_TICK_PERIOD_US    500000  // Timer1 tick = 500 ms
#define MAX_SERIAL_SILENCE_MS    1000    // Maximum serial silence between serial command characters = 1000 ms
#define MAX_SERIAL_SILENCE_TK    MAX_SERIAL_SILENCE_MS / (TIMER1_TICK_PERIOD_US / 1000)  // Max serial silence in ticks
#define BAUDRATE                 38400   // Serial port baudrate

#define BROADCAST                0x0A        // RF broadcast address
#define LOCALCALL                0x0B        // Local call address

// AT command set
#define AT_ATTENTION             "AT"        // Basic Attention command
#define AT_RESET                 "ATZ"       // Reset modem
#define AT_GOTO_CMDMODE          "+++"       // Go to command mode
#define AT_GOTO_DATAMODE         "ATO"       // Go to data mode
#define AT_FREQCHANNEL           "ATCH"      // Frequency channel
#define AT_SYNCWORD              "ATSW"      // Synchronization word
#define AT_DEVADDRESS            "ATDA"      // Device address
#define AT_ADDRCHECK             "ATAC"      // Address check
#define AT_RFSTATISTICS          "ATST"      // RF Statistics

// Packet types
// RF Packet types
#define PACKET_DATA_ASCII         0x01          // ASCII data packet
#define PACKET_DATA_INT           0x02          // Int16 data packet
#define PACKET_DATA_FLOAT         0x03          // Float data packet
#define PACKET_COMMAND            0x04          // Command packet
#define PACKET_REQUEST            0x05          // Request packet
// Serial Packet types
#define RF_PACKET                 0x01          // RF packet
#define COMMAND_PACKET            0x02          // Command packet
#define REQUEST_PACKET            0x03          // Request packet
// Request Types
#define REQ_VOLTAGE               0x01          // Battery Voltage Request
#define REQ_PING                  0x02          // Ping Request from RF
#define REQ_ADDRESS               0x03          // Address Reqest
// Command Types
#define CMD_REQUEST               0x01          // Send Request via RF
#define CMD_LED_ON                0x02          // Turn the System LED on
#define CMD_LED_OFF               0x03          // Turn the System LED off
#define CMD_RESET                 0x04          // Perform Software reset of System Controller
#define CMD_RXTXOFF               0x05          // Switches RX/TX-Pins to Input

// String to Int stuff
#define maxLength 128  
//Macros
#define enableINT0irq()    attachInterrupt(0, isrINT0event, FALLING);// digitalWrite(LEDPIN, LOW)
#define disableINT0irq()   detachInterrupt(0);// digitalWrite(LEDPIN, HIGH)
#define resetTimer()       t1Ticks = 0

// Serial modes
enum SERMODE
{
  SERMODE_DATA,
  SERMODE_COMMAND  
};

// Type of AT query
enum ATQUERY
{
  ATQUERY_COMMAND = 0,
  ATQUERY_REQUEST
};

// Global variables
char strSerial[SERIAL_BUF_LEN];          // Serial buffer
byte ch;
int len = 0;
SERMODE serMode = SERMODE_DATA;          // Serial mode (data or command mode)
byte t1Ticks = 0;                        // Timer 1 ticks
boolean packetAvailable = false;         // Wireless packet received when true
byte myAddress;
byte sendRFStatistics = false;
boolean debugMode = false;
byte syncWord [] = {0xAB,0xCD};
// CC1101 object
CC1101 cc1101;

#endif   

