#include "cc1101.h"
#include "TimerOne.h"
#include "EEPROM.h"
#include "calibration.h"
#include "systemService.h"

/**
 * Event on CC1011 GDO0 pin (INT0)
 *
 * Interrupt handler is executed when RF module
 * reports packet arrival.
 */
void isrINT0event(void)
{
  packetAvailable = true;
}

/**
 * Timer1 interrupt routine
 */
void isrT1event(void)
{
  if (t1Ticks == MAX_SERIAL_SILENCE_TK)
  {
    // Detach Timer1 interrupt
    Timer1.detachInterrupt();
    Timer1.stop();
    resetTimer();
    // Pending "+++" command?
    if (!strcmp(strSerial, AT_GOTO_CMDMODE))
    {
      disableINT0irq();  // Disable wireless reception interrupt
      Serial.println("OK-Command mode");
      serMode = SERMODE_COMMAND;
    }
    memset(strSerial, 0, sizeof(strSerial));
    len = 0;
    Timer1.resume();
  }
  else
    t1Ticks++;
}

/**
 * Arduino setup function
 *
 * Most of the actual setup happens in setupSystem(). 
 * This function is only required by the Arduino toolchain.
 */
void setup()
{
  setupSystem();
  // Enter RX state
  cc1101.setRxState();
  // Attach callback function for GDO0 (INT0)
  enableINT0irq();
  // Init Timer1
  Timer1.initialize(TIMER1_TICK_PERIOD_US);
}

/**
 * Arduino main loop
 *
 * Serial commands as well as RF packets are received.
 * RF packets call handlePacket() and serial commands
 * call handleSerialCmd()
 *
 * \todo Self diagnostics?
 * \todo discovery of other nodes
 */
void loop()
{
  // Read wireless packet?
  if (packetAvailable)
  {
    // Disable wireless reception interrupt
    disableINT0irq();
    byte i;
    CCPACKET packet;
    packetAvailable = false;
    if (cc1101.receiveData(&packet) > 0)
    {
      dMsg("RF Packet received.");
      handlePacket(packet);
    }
    // Enable wireless reception interrupt
    enableINT0irq(); 
  }
  // Read serial command?
  if (Serial.available() > 0)
  {
    // Disable wireless reception interrupt
    disableINT0irq();

    ch = Serial.read();

    if (len >= SERIAL_BUF_LEN-1)
    {
      memset(strSerial, 0, sizeof(strSerial));
      len = 0;
    }
    else if (ch == 0x0D) 
    {
      dMsg("Received Serial String.");
      Timer1.detachInterrupt();
      strSerial[len] = 0;
      handleSerialCmd(strSerial);
      memset(strSerial, 0, sizeof(strSerial));
      len = 0;
    }
    else
    {
      strSerial[len] = ch; 
      len++;
      resetTimer();
      // Attach interrupt function to Timer1
      Timer1.attachInterrupt(isrT1event);
    }
    // Enable wireless reception interrupt
    enableINT0irq();
  }
}
















